/*
 * A registry of objects supporting notifications.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <map>
#include <mutex>

#include <fsserver/file_notification.h>
#include <mem/types.h>



/* Mapping type from file identifiers to notification managers. */

typedef std::map<fileid_t, FileNotification *> FileNotifiers;
typedef std::map<fileid_t, FileNotification *> FileNotifierEntry;



/* A registry of directory access objects. */

class FileNotifierRegistry
{
protected:
    FileNotifiers _notifiers;
    std::mutex _lock;

    FileNotification *_get(fileid_t fileid);

public:
    explicit FileNotifierRegistry();

    void attach(fileid_t fileid);

    void detach(fileid_t fileid);

    FileNotification *get(fileid_t fileid);
};

// vim: tabstop=4 expandtab shiftwidth=4
